@@ -7,10 +7,12 @@ if (typeof fetch === 'function') {
     fetchApi = global.fetch
   } else if (typeof window !== 'undefined' && window.fetch) {
     fetchApi = window.fetch
+  } else {
+    fetchApi = fetch
   }
 }
 let XmlHttpRequestApi
-if (hasXMLHttpRequest) {
+if (hasXMLHttpRequest()) {
   if (typeof global !== 'undefined' && global.XMLHttpRequest) {
     XmlHttpRequestApi = global.XMLHttpRequest
   } else if (typeof window !== 'undefined' && window.XMLHttpRequest) {
@@ -41,6 +43,17 @@ const addQueryString = (url, params) => {
   return url
 }
 
+const fetchIt = (url, fetchOptions, callback) => {
+  fetchApi(url, fetchOptions).then((response) => {
+    if (!response.ok) return callback(response.statusText || 'Error', { status: response.status })
+    response.text().then((data) => {
+      callback(null, { status: response.status, data })
+    }).catch(callback)
+  }).catch(callback)
+}
+
+let omitFetchOptions = false
+
 // fetch api stuff
 const requestWithFetch = (options, url, payload, callback) => {
   if (options.queryStringParams) {
@@ -48,16 +61,28 @@ const requestWithFetch = (options, url, payload, callback) => {
   }
   const headers = defaults({}, typeof options.customHeaders === 'function' ? options.customHeaders() : options.customHeaders)
   if (payload) headers['Content-Type'] = 'application/json'
-  fetchApi(url, defaults({
+  const reqOptions = typeof options.requestOptions === 'function' ? options.requestOptions(payload) : options.requestOptions
+  const fetchOptions = defaults({
     method: payload ? 'POST' : 'GET',
     body: payload ? options.stringify(payload) : undefined,
     headers
-  }, typeof options.requestOptions === 'function' ? options.requestOptions(payload) : options.requestOptions)).then((response) => {
-    if (!response.ok) return callback(response.statusText || 'Error', { status: response.status })
-    response.text().then((data) => {
-      callback(null, { status: response.status, data })
-    }).catch(callback)
-  }).catch(callback)
+  }, omitFetchOptions ? {} : reqOptions)
+  try {
+    fetchIt(url, fetchOptions, callback)
+  } catch (e) {
+    if (!reqOptions || Object.keys(reqOptions).length === 0 || !e.message || e.message.indexOf('not implemented') < 0) {
+      return callback(e)
+    }
+    try {
+      Object.keys(reqOptions).forEach((opt) => {
+        delete fetchOptions[opt]
+      })
+      fetchIt(url, fetchOptions, callback)
+      omitFetchOptions = true
+    } catch (err) {
+      callback(err)
+    }
+  }
 }
 
 // xml http request stuff
@@ -118,10 +143,12 @@ const request = (options, url, payload, callback) => {
     return requestWithFetch(options, url, payload, callback)
   }
 
-  if (hasXMLHttpRequest || typeof ActiveXObject === 'function') {
+  if (hasXMLHttpRequest() || typeof ActiveXObject === 'function') {
     // use xml http request
     return requestWithXmlHttpRequest(options, url, payload, callback)
   }
+
+  callback(new Error('No fetch and no xhr implementation found!'))
 }
 
 export default request
